@@ -211,6 +211,20 @@ class DeferringChunkedProducerTests(unittest.TestCase):
         producer = self._makeOne(wrapped, footers=['a', 'b'])
         self.assertEqual(producer.more(), '0\r\na\r\nb\r\n\r\n')
 
+    def test_more_nodata_footers(self):
+        wrapped = DummyProducer('')
+        producer = self._makeOne(wrapped, footers=['a', 'b'])
+        self.assertEqual(producer.more(), '0\r\na\r\nb\r\n\r\n')
+
+    def test_more_nodata_nofooters(self):
+        wrapped = DummyProducer('')
+        producer = self._makeOne(wrapped)
+        self.assertEqual(producer.more(), '0\r\n\r\n')
+
+    def test_more_noproducer(self):
+        producer = self._makeOne(None)
+        self.assertEqual(producer.more(), '')
+
 class DeferringCompositeProducerTests(unittest.TestCase):
     def _getTargetClass(self):
         from supervisor.http import deferring_composite_producer
@@ -297,6 +311,177 @@ class DeferringHookedProducerTests(unittest.TestCase):
         self.assertEqual(producer.more(), '')
         self.assertEqual(L, [0])
 
+    def test_more_noproducer(self):
+        producer = self._makeOne(None, None)
+        self.assertEqual(producer.more(), '')
+
+class DeferringHttpRequestTests(unittest.TestCase):
+    def _getTargetClass(self):
+        from supervisor.http import deferring_http_request
+        return deferring_http_request
+
+    def _makeOne(
+        self,
+        channel=None,
+        req='GET / HTTP/1.0',
+        command='GET',
+        uri='/',
+        version='1.0',
+        header=(),
+        ):
+        return self._getTargetClass()(
+            channel, req, command, uri, version, header
+            )
+
+    def _makeChannel(self):
+        class Channel:
+            closed = False
+            def close_when_done(self):
+                self.closed = True
+            def push_with_producer(self, producer):
+                self.producer = producer
+        return Channel()
+
+    def test_done_http_10_nokeepalive(self):
+        channel = self._makeChannel()
+        inst = self._makeOne(channel=channel, version='1.0')
+        inst.done()
+        self.assertTrue(channel.closed)
+
+    def test_done_http_10_keepalive_no_content_length(self):
+        channel = self._makeChannel()
+        inst = self._makeOne(
+            channel=channel,
+            version='1.0',
+            header=['Connection: Keep-Alive'],
+            )
+
+        inst.done()
+        self.assertTrue(channel.closed)
+
+    def test_done_http_10_keepalive_and_content_length(self):
+        channel = self._makeChannel()
+        inst = self._makeOne(
+            channel=channel,
+            version='1.0',
+            header=['Connection: Keep-Alive'],
+            )
+        inst.reply_headers['Content-Length'] = 1
+        inst.done()
+        self.assertEqual(inst['Connection'], 'Keep-Alive')
+        self.assertFalse(channel.closed)
+
+    def test_done_http_11_connection_close(self):
+        channel = self._makeChannel()
+        inst = self._makeOne(
+            channel=channel,
+            version='1.1',
+            header=['Connection: close']
+            )
+        inst.done()
+        self.assertTrue(channel.closed)
+
+    def test_done_http_11_unknown_transfer_encoding(self):
+        channel = self._makeChannel()
+        inst = self._makeOne(
+            channel=channel,
+            version='1.1',
+            )
+        inst.reply_headers['Transfer-Encoding'] = 'notchunked'
+        inst.done()
+        self.assertTrue(channel.closed)
+
+    def test_done_http_11_chunked_transfer_encoding(self):
+        channel = self._makeChannel()
+        inst = self._makeOne(
+            channel=channel,
+            version='1.1',
+            )
+        inst.reply_headers['Transfer-Encoding'] = 'chunked'
+        inst.done()
+        self.assertFalse(channel.closed)
+
+    def test_done_http_11_use_chunked(self):
+        channel = self._makeChannel()
+        inst = self._makeOne(
+            channel=channel,
+            version='1.1',
+            )
+        inst.use_chunked = True
+        inst.done()
+        self.assertTrue('Transfer-Encoding' in inst)
+        self.assertFalse(channel.closed)
+
+    def test_done_http_11_wo_content_length_no_te_no_use_chunked_close(self):
+        channel = self._makeChannel()
+        inst = self._makeOne(
+            channel=channel,
+            version='1.1',
+            )
+        inst.use_chunked = False
+        inst.done()
+        self.assertTrue(channel.closed)
+
+    def test_done_http_09(self):
+        channel = self._makeChannel()
+        inst = self._makeOne(
+            channel=channel,
+            version=None,
+            )
+        inst.done()
+        self.assertTrue(channel.closed)
+
+class DeferringHttpChannelTests(unittest.TestCase):
+    def _getTargetClass(self):
+        from supervisor.http import deferring_http_channel
+        return deferring_http_channel
+
+    def _makeOne(self):
+        return self._getTargetClass()(
+            server=None,
+            conn=None,
+            addr=None
+            )
+
+    def test_defaults_delay_and_last_writable_check_time(self):
+        channel = self._makeOne()
+        self.assertEqual(channel.delay, 0)
+        self.assertTrue(channel.last_writable_check > 0)
+
+    def test_writable_with_delay_is_False_if_elapsed_lt_delay(self):
+        channel = self._makeOne()
+        channel.delay = 2
+        channel.last_writable_check = _NOW
+        later = _NOW + 1
+        self.assertFalse(channel.writable(now=later))
+        self.assertEqual(channel.last_writable_check, _NOW)
+
+    def test_writable_with_delay_is_False_if_elapsed_eq_delay(self):
+        channel = self._makeOne()
+        channel.delay = 2
+        channel.last_writable_check = _NOW
+        later = _NOW + channel.delay
+        self.assertFalse(channel.writable(now=later))
+        self.assertEqual(channel.last_writable_check, _NOW)
+
+    def test_writable_with_delay_is_True_if_elapsed_gt_delay(self):
+        channel = self._makeOne()
+        channel.delay = 2
+        channel.last_writable_check = _NOW
+        later = _NOW + channel.delay + 0.1
+        self.assertTrue(channel.writable(now=later))
+        self.assertEqual(channel.last_writable_check, later)
+
+    def test_writable_with_delay_is_True_if_system_time_goes_backwards(self):
+        channel = self._makeOne()
+        channel.delay = 2
+        channel.last_writable_check = _NOW
+        later = _NOW - 3600 # last check was in the future
+        self.assertTrue(channel.writable(now=later))
+        self.assertEqual(channel.last_writable_check, later)
+
+_NOW = 1470085990
+
 class EncryptedDictionaryAuthorizedTests(unittest.TestCase):
     def _getTargetClass(self):
         from supervisor.http import encrypted_dictionary_authorizer
