@@ -22,6 +22,7 @@ class Faults:
     SIGNATURE_UNSUPPORTED = 4
     SHUTDOWN_STATE = 6
     BAD_NAME = 10
+    BAD_SIGNAL = 11
     NO_FILE = 20
     NOT_EXECUTABLE = 21
     FAILED = 30
@@ -47,6 +48,9 @@ class RPCError(Exception):
         if extra is not None:
             self.text = '%s: %s' % (self.text, extra)
 
+    def __str__(self):
+        return 'code=%r, text=%r' % (self.code, self.text)
+
 class DeferredXMLRPCResponse:
     """ A medusa producer that implements a deferred callback; requires
     a subclass of asynchat.async_chat that handles NOT_DONE_YET sentinel """
@@ -76,8 +80,10 @@ class DeferredXMLRPCResponse:
             return self.getresponse(body)
 
         except:
-            # report unexpected exception back to server
-            traceback.print_exc()
+            tb = traceback.format_exc()
+            self.request.channel.server.logger.log(
+                "XML-RPC response callback error", tb
+                )
             self.finished = True
             self.request.error(500)
 
@@ -233,7 +239,7 @@ class SystemNamespaceRPCInterface:
         """Process an array of calls, and return an array of
         results. Calls should be structs of the form {'methodName':
         string, 'params': array}. Each result will either be a
-        single-item array containg the result value, or a struct of
+        single-item array containing the result value, or a struct of
         the form {'faultCode': int, 'faultString': string}. This is
         useful when you need to make lots of small calls without lots
         of round trips.
@@ -241,69 +247,100 @@ class SystemNamespaceRPCInterface:
         @param array calls  An array of call requests
         @return array result  An array of results
         """
-        producers = []
+        remaining_calls = calls[:] # [{'methodName':x, 'params':x}, ...]
+        callbacks = [] # always empty or 1 callback function only
+        results = [] # results of completed calls
 
-        for call in calls:
-            try:
-                name = call['methodName']
-                params = call.get('params', [])
-                if name == 'system.multicall':
-                    # Recursive system.multicall forbidden
-                    raise RPCError(Faults.INCORRECT_PARAMETERS)
-                root = AttrDict(self.namespaces)
-                value = traverse(root, name, params)
-            except RPCError, inst:
-                value = {'faultCode': inst.code,
-                         'faultString': inst.text}
-            except:
-                errmsg = "%s:%s" % (sys.exc_type, sys.exc_value)
-                value = {'faultCode': 1, 'faultString': errmsg}
-            producers.append(value)
+        # args are only to fool scoping and are never passed by caller
+        def multi(remaining_calls=remaining_calls,
+                  callbacks=callbacks,
+                  results=results):
 
-        results = []
-
-        def multiproduce():
-            """ Run through all the producers in order """
-            if not producers:
-                return []
-
-            callback = producers.pop(0)
-
-            if isinstance(callback, types.FunctionType):
+            # if waiting on a callback, call it, then remove it if it's done
+            if callbacks:
                 try:
-                    value = callback()
-                except RPCError, inst:
-                    value = {'faultCode':inst.code, 'faultString':inst.text}
+                    value = callbacks[0]()
+                except RPCError, exc:
+                    value = {'faultCode': exc.code,
+                             'faultString': exc.text}
+                except:
+                    info = sys.exc_info()
+                    errmsg = "%s:%s" % (info[0], info[1])
+                    value = {'faultCode': Faults.FAILED,
+                             'faultString': 'FAILED: ' + errmsg}
+                if value is not NOT_DONE_YET:
+                    callbacks.pop(0)
+                    results.append(value)
 
-                if value is NOT_DONE_YET:
-                    # push it back in the front of the queue because we
-                    # need to finish the calls in requested order
-                    producers.insert(0, callback)
-                    return NOT_DONE_YET
-            else:
-                value = callback
+            # if we don't have a callback now, pop calls and call them in
+            # order until one returns a callback.
+            while (not callbacks) and remaining_calls:
+                call = remaining_calls.pop(0)
+                name = call.get('methodName', None)
+                params = call.get('params', [])
 
-            results.append(value)
+                try:
+                    if name is None:
+                        raise RPCError(Faults.INCORRECT_PARAMETERS,
+                            'No methodName')
+                    if name == 'system.multicall':
+                        raise RPCError(Faults.INCORRECT_PARAMETERS,
+                            'Recursive system.multicall forbidden')
+                    # make the call, may return a callback or not
+                    root = AttrDict(self.namespaces)
+                    value = traverse(root, name, params)
+                except RPCError, exc:
+                    value = {'faultCode': exc.code,
+                             'faultString': exc.text}
+                except:
+                    info = sys.exc_info()
+                    errmsg = "%s:%s" % (info[0], info[1])
+                    value = {'faultCode': Faults.FAILED,
+                             'faultString': 'FAILED: ' + errmsg}
 
-            if producers:
-                # only finish when all producers are finished
+                if isinstance(value, types.FunctionType):
+                    callbacks.append(value)
+                else:
+                    results.append(value)
+
+            # we are done when there's no callback and no more calls queued
+            if callbacks or remaining_calls:
                 return NOT_DONE_YET
+            else:
+                return results
+        multi.delay = 0.05
 
-            return results
-
-        multiproduce.delay = .05
-        return multiproduce
+        # optimization: multi() is called here instead of just returning
+        # multi in case all calls complete and we can return with no delay.
+        value = multi()
+        if value is NOT_DONE_YET:
+            return multi
+        else:
+            return value
 
 class AttrDict(dict):
     # hack to make a dict's getattr equivalent to its getitem
     def __getattr__(self, name):
-        return self[name]
+        return self.get(name)
 
 class RootRPCInterface:
     def __init__(self, subinterfaces):
         for name, rpcinterface in subinterfaces:
             setattr(self, name, rpcinterface)
 
+def capped_int(value):
+    i = int(value)
+    if i < xmlrpclib.MININT:
+        i = xmlrpclib.MININT
+    elif i > xmlrpclib.MAXINT:
+        i = xmlrpclib.MAXINT
+    return i
+
+def make_datetime(text):
+    return datetime.datetime(
+        *time.strptime(text, "%Y%m%dT%H:%M:%S")[:6]
+    )
+
 class supervisor_xmlrpc_handler(xmlrpc_handler):
     path = '/RPC2'
     IDENT = 'Supervisor XML-RPC Handler'
@@ -322,16 +359,26 @@ class supervisor_xmlrpc_handler(xmlrpc_handler):
     def match(self, request):
         return request.uri.startswith(self.path)
 
-    def continue_request (self, data, request):
+    def continue_request(self, data, request):
         logger = self.supervisord.options.logger
 
         try:
-
-            params, method = self.loads(data)
+            try:
+                params, method = self.loads(data)
+            except:
+                logger.error(
+                    'XML-RPC request data %r is invalid: unmarshallable' %
+                    (data,)
+                )
+                request.error(400)
+                return
 
             # no <methodName> in the request or name is an empty string
             if not method:
-                logger.trace('XML-RPC request received with no method name')
+                logger.error(
+                    'XML-RPC request data %r is invalid: no method name' %
+                    (data,)
+                    )
                 request.error(400)
                 return
 
@@ -343,14 +390,6 @@ class supervisor_xmlrpc_handler(xmlrpc_handler):
             try:
                 logger.trace('XML-RPC method called: %s()' % method)
                 value = self.call(method, params)
-                # application-specific: instead of we never want to
-                # marshal None (even though we could by saying allow_none=True
-                # in dumps within xmlrpc_marshall), this is meant as
-                # a debugging fixture, see issue 223.
-                assert value is not None, (
-                    'return value from method %r with params %r is None' %
-                    (method, params)
-                    )
                 logger.trace('XML-RPC method %s() returned successfully' %
                              method)
             except RPCError, err:
@@ -377,10 +416,11 @@ class supervisor_xmlrpc_handler(xmlrpc_handler):
                 request.done()
 
         except:
-            io = StringIO()
-            traceback.print_exc(file=io)
-            val = io.getvalue()
-            logger.critical(val)
+            tb = traceback.format_exc()
+            logger.critical(
+                "Handling XML-RPC request with data %r raised an unexpected "
+                "exception: %s" % (data, tb)
+                )
             # internal error, report as HTTP server error
             request.error(500)
 
@@ -388,27 +428,18 @@ class supervisor_xmlrpc_handler(xmlrpc_handler):
         return traverse(self.rpcinterface, method, params)
 
 def traverse(ob, method, params):
-    dotted_parts = method.split('.')
-    # security (CVE-2017-11610, don't allow object traversal)
-    if len(dotted_parts) != 2:
-        raise RPCError(Faults.UNKNOWN_METHOD)
-    namespace, method = dotted_parts
-
-    # security (don't allow methods that start with an underscore to
-    # be called remotely)
-    if method.startswith('_'):
-        raise RPCError(Faults.UNKNOWN_METHOD)
-
-    rpcinterface = getattr(ob, namespace, None)
-    if rpcinterface is None:
-        raise RPCError(Faults.UNKNOWN_METHOD)
-
-    func = getattr(rpcinterface, method, None)
-    if not isinstance(func, types.MethodType):
-        raise RPCError(Faults.UNKNOWN_METHOD)
+    path = method.split('.')
+    for name in path:
+        if name.startswith('_'):
+            # security (don't allow things that start with an underscore to
+            # be called remotely)
+            raise RPCError(Faults.UNKNOWN_METHOD)
+        ob = getattr(ob, name, None)
+        if ob is None:
+            raise RPCError(Faults.UNKNOWN_METHOD)
 
     try:
-        return func(*params)
+        return ob(*params)
     except TypeError:
         raise RPCError(Faults.INCORRECT_PARAMETERS)
 
@@ -548,11 +579,6 @@ if iterparse is not None:
     import datetime, time
     from base64 import decodestring
 
-    def make_datetime(text):
-        return datetime.datetime(
-            *time.strptime(text, "%Y%m%dT%H:%M:%S")[:6]
-        )
-
     unmarshallers = {
         "int": lambda x: int(x.text),
         "i4": lambda x: int(x.text),
